@@ -17,20 +17,20 @@ NumericVector bucket_fill(NumericVector im,int x,int y,int z,NumericVector color
 }
 
 // [[Rcpp::export]]
-NumericVector bucket_select(NumericVector im,int x,int y,int z,float sigma=0,bool high_connexity=false)
+LogicalVector bucket_select(NumericVector im,int x,int y,int z,float sigma=0,bool high_connexity=false)
 {
   CId img = as<CId >(im);
-  CIb out;
-  NumericVector v; //container for future output
+  CIb out(img,"xyzc");
+  NumericVector color(img.spectrum());
+  float opacity=1;
+
   try{
-    NumericVector color(img.spectrum());
-    float opacity=1;
     img.draw_fill(x-1,y-1,z-1,color.begin(),opacity,out,sigma,high_connexity);
-    v = wrap(out);
+  
 
   }
   catch(CImgException &e){
     forward_exception_to_r(e);
   }
-  return v;
+  return wrap(out);
 }
